# Dialogs.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2001   Michele Campeotto <micampe@f2s.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import Moleskine
import findtools
from GladeDialog import *
import GTKSCINTILLA

find_dialog = None
replace_dialog = None
options_dialog = None
langs_dialog = None

def _parse_color(color):
    import types
    if type(color) == types.IntType:
        b = (color & 0xFF0000) >> 16
        g = (color & 0x00FF00) >> 8
        r = (color & 0x0000FF)
        return (r, g, b)
    elif type(color) == types.TupleType:
        return color[2] * 65536 + color[1] * 256 + color[0]
    else:
        raise TypeError, '`color\' must be tuple or integer.'

class OptionsDialog(GladeDialog):
    def __init__(self):
        GladeDialog.__init__(self, 'options_dialog')
        self.set_transient_for(Moleskine.app)
        self.connect('clicked', self.clicked)
        self.connect('destroy', self.destroy)
        
        self.digits_fixed.connect('toggled', self.digits_fixed_toggled)
        
        prefs = Moleskine.app.prefs
        
    # History
        self.save_win_size.set_active(int(prefs['geometry/save_win_size']))
        self.mru_size.set_value(int(prefs['history/mru_size']))
        self.mru_sort.set_active(int(prefs['history/mru_sort']))
    # Default style
        self.def_font.set_font_name(prefs['default style/font'])
        (r, g, b) = _parse_color(int(prefs['default style/fore']))
        self.def_fore.set_i8(r, g, b, 0xFF)
        (r, g, b) = _parse_color(int(prefs['default style/back']))
        self.def_back.set_i8(r, g, b, 0xFF)
    # Line numbers style
        self.linen_font.set_font_name(prefs['line numbers/font'])
        (r, g, b) = _parse_color(int(prefs['line numbers/fore']))
        self.linen_fore.set_i8(r, g, b, 0xFF)
        (r, g, b) = _parse_color(int(prefs['line numbers/back']))
        self.linen_back.set_i8(r, g, b, 0xFF)
    # Selection color
        (r, g, b) = _parse_color(int(prefs['default style/selection']))
        self.selection_colour.set_i8(r, g, b, 0xFF)
    # Bookmark color
        (r, g, b) = _parse_color(int(prefs['default style/bookmark']))
        self.bookmark_colour.set_i8(r, g, b, 0xFF)
    # Keyboard
        keys = int(prefs['keyboard/keybindings'])
        if keys == 2:
            self.keys_emacs.set_active(TRUE)
        elif keys == 1:
            self.keys_gnome.set_active(TRUE)
        else:
            self.keys_moleskine.set_active(TRUE)
    # Line numbers digits
        self.show_linen.set_active(int(prefs['line numbers/show']))
        if int(prefs['line numbers/auto']):
            self.digits_auto.set_active(TRUE)
            self.digits_num.set_sensitive(FALSE)
        else:
            self.digits_fixed.set_active(TRUE)
            self.digits_num.set_sensitive(TRUE)
        self.digits_num.set_value(int(prefs['line numbers/digits']))
    
    def digits_fixed_toggled(self, w, user_data=None):
        if w.get_active():
            self.digits_num.set_sensitive(TRUE)
        else:
            self.digits_num.set_sensitive(FALSE)
    
    def apply(self):
        from string import split
        
        prefs = Moleskine.app.prefs
        
    # History
        prefs['geometry/save_win_size'] = self.save_win_size.get_active()
        prefs['history/mru_size'] = self.mru_size.get_value_as_int()
        prefs['history/mru_sort'] = self.mru_sort.get_active()
    # Default style
        prefs['default style/font'] = self.def_font.get_font_name()
        prefs['default style/fore'] = \
                _parse_color(self.def_fore.get_i8())
        prefs['default style/back'] = \
                _parse_color(self.def_back.get_i8())
    # Line numbers style
        prefs['line numbers/font'] = self.linen_font.get_font_name()
        prefs['line numbers/fore'] = \
                _parse_color(self.linen_fore.get_i8())
        prefs['line numbers/back'] = \
                _parse_color(self.linen_back.get_i8())
    # Selection color
        prefs['default style/selection'] = \
                _parse_color(self.selection_colour.get_i8())
    # Bookmark color
        prefs['default style/bookmark'] = \
                _parse_color(self.bookmark_colour.get_i8())
    # Keyboard
        if self.keys_emacs.get_active():
            prefs['keyboard/keybindings'] = 2
        elif self.keys_gnome.get_active():
            prefs['keyboard/keybindings'] = 1
        else:
            prefs['keyboard/keybindings'] = 0
    # Line numbers digits
        prefs['line numbers/show'] = self.show_linen.get_active()
        prefs['line numbers/auto'] = self.digits_auto.get_active()
        prefs['line numbers/digits'] = self.digits_num.get_value_as_int()
        
        for doc in Moleskine.app.documents:
            doc.setup_line_numbers_digits()
        Moleskine.app.update_mru_menu()
        Moleskine.app.update_styles()
        Moleskine.app.prefs.save()
    
    def destroy(self, w, user_data=None):
        global options_dialog
        options_dialog = None
    
    def clicked(self, w, button, user_data=None):
        if button == 0:
            self.apply()
            self.close()
        elif button == 1:
            self.apply()
        else:
            self.close()

class LanguagesDialog(GladeDialog):
    def __init__(self):
        GladeDialog.__init__(self, 'languages_dialog')
        self.set_transient_for(Moleskine.app)
        self.connect('clicked', self.clicked)
        self.connect('destroy', self.destroy)
        
        self.styles_list.set_auto_sort(TRUE)
        
        lang_names = Moleskine.app.langs.keys()
        lang_names.sort()
        self.lang_name_combo.set_popdown_strings(lang_names)
        self.lang_name.connect('changed', self.language_changed)
        
        import Languages
        lexers = Languages.Lexers.keys()
        lexers.sort()
        self.lexer_combo.set_popdown_strings(lexers)
        self.lexer.connect('changed', self.lexer_changed)
        self.styles_list.connect('select_row', self.style_selected)
        self.styles_list.connect('unselect_row', self.style_unselected)
        self.style_reset.connect('clicked', self.reset_style_clicked)
        
        self.rules_list.connect('select_row', self.rule_selected)
        self.rules_list.connect('unselect_row', self.rule_unselected)
        self.new_rule.connect('clicked', self.new_rule_clicked)
        self.del_rule.connect('clicked', self.del_rule_clicked)
        self.rule_entry.connect('changed', self.edit_rule)
        self.rule_glob.connect('clicked', self.set_rule_type, 0)
        self.rule_name_re.connect('clicked', self.set_rule_type, 1)
        self.rule_text_re.connect('clicked', self.set_rule_type, 2)
        
        lang = Moleskine.app.get_current_document().language
        self.lang_name.set_text(lang.name)
    
    def setup_editor_tab(self, lang):
        self.tab_size.set_value(lang.tab_size)
        self.soft_tabs.set_active(lang.soft_tabs)
        self.show_indent.set_active(lang.show_indent)
        self.indent_size.set_value(lang.indent_size)
        self.auto_indent.set_active(lang.auto_indent)
        if lang.edge_indicator == 2:
            self.edge_background.set_active(TRUE)
        elif lang.edge_indicator == 1:
            self.edge_line.set_active(TRUE)
        else:
            self.edge_none.set_active(TRUE)
        self.edge_column.set_value(lang.edge_column)
        (r, g, b) = _parse_color(lang.edge_color)
        self.edge_color.set_i8(r, g, b, 0xFF)
    
    def setup_highlight_tab(self, lang):
        import Languages
        
        for l in Languages.Lexers.keys():
            if Languages.Lexers[l] == lang.lexer:
                lexer_name = l
                break
        self.lexer.set_text(lexer_name)
    
    def setup_keywords_tab(self, lang):
        for kwclass in range(5):
            if lang.keywords[kwclass] is not None:
                kwlist = self['keywords%i_list' % (kwclass + 1)]
                kwlist.freeze()
                kwlist.delete_text(0, -1)
                kwlist.insert_defaults(lang.keywords[kwclass])
                kwlist.thaw()
    
    def setup_ftypes_tab(self, lang):
        self.rules_list.clear()
        self.rules_list.freeze()
        for rule in lang.rules:
            if rule[0] == 0:
                rule_type = 'Shell glob'
            elif rule[0] == 1:
                rule_type = 'Filename regexp'
            else:
                rule_type = 'Text regexp'
            row = self.rules_list.append((rule_type, rule[1]))
            self.rules_list.set_row_data(row, rule[0])
        self.rules_list.thaw()
    
    def language_changed(self, w, user_data=None):
        lang_name = self.lang_name.get_text()
        lang = Moleskine.app.langs[lang_name]
        
        self.setup_editor_tab(lang)
        self.setup_highlight_tab(lang)
        self.setup_keywords_tab(lang)
        self.setup_ftypes_tab(lang)
    
    def lexer_changed(self, w, data=None):
        import Languages, copy
        
        lang_name = self.lang_name.get_text()
        lang = Moleskine.app.langs[lang_name]
        lexer_name = self.lexer.get_text()
        lexer = Languages.Lexers[lexer_name]
        
        self.styles_list.unselect_all()
        self.styles_list.clear()
        self.styles_list.freeze()
        if Languages.Styles.has_key(lexer):
            styles = Languages.Styles[lexer]
            for s in styles.keys():
                row = self.styles_list.append((s,))
                if styles.has_key(s) and lang.styles.has_key(styles[s]):
                    self.styles_list.set_row_data(row,
                            (styles[s], copy.deepcopy(lang.styles[styles[s]])))
        self.styles_list.thaw()
    
    def style_selected(self, clist, row, column, event, user_data=None):
        def merge_font(font):
            import string
            
            df = string.split(Moleskine.app.prefs['default style/font'], '-')
            sf = string.split(font, '-')
            for i in range(min(len(df), len(sf))):
                if sf[i] == '*':
                    sf[i] = df[i]
            return string.join(sf, '-')
        
        style = clist.get_row_data(row)
        if style is None:
            return
        
        style = style[1]
        
        if style['font'] is not None:
            font = merge_font(style['font'])
        else:
            font = Moleskine.app.prefs['default style/font']
        self.style_font.set_font_name(font)
        if style['fore'] is not None:
            r, g, b = _parse_color(int(style['fore']))
        else:
            r, g, b = _parse_color(int(
                    Moleskine.app.prefs['default style/fore']))
        self.style_fore.set_i8(r, g, b, 0xFF)
        if style['back'] is not None:
            r, g, b = _parse_color(int(style['back']))
        else:
            r, g, b = _parse_color(int(
                    Moleskine.app.prefs['default style/back']))
        self.style_back.set_i8(r, g, b, 0xFF)

        self.style_font.set_sensitive(TRUE)
        self.style_fore.set_sensitive(TRUE)
        self.style_back.set_sensitive(TRUE)
        self.style_reset.set_sensitive(TRUE)
    
    def style_unselected(self, clist, row, column, event, user_data=None):
        def unmerge_font(font):
            import string
            
            df = string.split(Moleskine.app.prefs['default style/font'], '-')
            sf = string.split(font, '-')
            for i in range(min(len(df), len(sf))):
                if sf[i] == df[i]:
                    sf[i] = '*'
            return string.join(sf, '-')
        
        style = clist.get_row_data(row)
        if style is None:
            return
        
        style = style[1]
        
        style['font'] = unmerge_font(self.style_font.get_font_name())
        style['fore'] = str(_parse_color(self.style_fore.get_i8()))
        style['back'] = str(_parse_color(self.style_back.get_i8()))
        
        self.style_font.set_sensitive(FALSE)
        self.style_fore.set_sensitive(FALSE)
        self.style_back.set_sensitive(FALSE)
        self.style_reset.set_sensitive(FALSE)
    
    def reset_style_clicked(self, w, user_data=None):
        row = self.styles_list.selection[0]
        style = self.styles_list.get_row_data(row)
        style[1]['font'] = None
        style[1]['fore'] = None
        style[1]['back'] = None
        self.styles_list.select_row(row, 0)
    
    def new_rule_clicked(self, w, user_data=None):
        row = self.rules_list.append(('Shell glob', ''))
        self.rules_list.set_row_data(row, 0)
        self.rules_list.select_row(row, 0)
        self.rule_entry.set_text('')
        self.rule_entry.grab_focus()
    
    def del_rule_clicked(self, w, user_data=None):
        self.rules_list.remove(self.rules_list.selection[0])
    
    def edit_rule(self, w, user_data=None):
        if len(self.rules_list.selection) > 0:
            row = self.rules_list.selection[0]
            self.rules_list.set_text(row, 1, w.get_text())
    
    def set_rule_type(self, w, user_data=None):
        if w.get_active() == TRUE:
            row = self.rules_list.selection[0]
            if user_data == 0:
                self.rules_list.set_text(row, 0, 'Shell glob')
            elif user_data == 1:
                self.rules_list.set_text(row, 0, 'Filename regexp')
            else:
                self.rules_list.set_text(row, 0, 'Text regexp')
    
    def rule_selected(self, clist, row, column, event, user_data=None):
        rule = clist.get_text(row, 1)
        self.rule_entry.set_text(rule)
        rule_type = clist.get_row_data(row)
        if rule_type == 0:
            self.rule_glob.set_active(TRUE)
        elif  rule_type == 1:
            self.rule_name_re.set_active(TRUE)
        else:
            self.rule_text_re.set_active(TRUE)
        
        self.rule_entry.set_sensitive(TRUE)
        self.del_rule.set_sensitive(TRUE)
        self.rule_glob.set_sensitive(TRUE)
        self.rule_name_re.set_sensitive(TRUE)
        self.rule_text_re.set_sensitive(TRUE)
    
    def rule_unselected(self, clist, row, column, event, user_data=None):
        self.rule_entry.set_text('')
        self.rule_glob.set_active(TRUE)
        
        self.rule_entry.set_sensitive(FALSE)
        self.del_rule.set_sensitive(FALSE)
        self.rule_glob.set_sensitive(FALSE)
        self.rule_name_re.set_sensitive(FALSE)
        self.rule_text_re.set_sensitive(FALSE)
    
    def apply(self):
        import Languages, copy
        
        langs = Moleskine.app.langs
        lang_name = self.lang_name.get_text()
        lang = langs[lang_name]
        
        lang.tab_size = self.tab_size.get_value_as_int()
        lang.soft_tabs = self.soft_tabs.get_active()
        lang.indent_size = self.indent_size.get_value_as_int()
        lang.show_indent = self.show_indent.get_active()
        lang.auto_indent = self.auto_indent.get_active()
        if self.edge_none.get_active():
            lang.edge_indicator = 0
        elif self.edge_line.get_active():
            lang.edge_indicator = 1
        else:
            lang.edge_indicator = 2
        lang.edge_column = self.edge_column.get_value_as_int()
        lang.edge_color = _parse_color(self.edge_color.get_i8())
        
        if len(self.styles_list.selection) > 0:
            self.styles_list.unselect_all()
        lang.lexer = Languages.Lexers[self.lexer.get_text()]
        for row in range(self.styles_list.rows):
            style = self.styles_list.get_row_data(row)
            if style is None:
                continue
            lang.styles[style[0]] = copy.deepcopy(style[1])
        
        lang.keywords = [None, None, None, None, None]
        lang.keywords[0] = self.keywords1_list.get_chars(0, -1)
        lang.keywords[1] = self.keywords2_list.get_chars(0, -1)
        lang.keywords[2] = self.keywords3_list.get_chars(0, -1)
        lang.keywords[3] = self.keywords4_list.get_chars(0, -1)
        lang.keywords[4] = self.keywords5_list.get_chars(0, -1)
        
        lang.rules = []
        for row in range(self.rules_list.rows):
            type = self.rules_list.get_row_data(row)
            rule = self.rules_list.get_text(row, 1)
            lang.rules.append((type, rule))
        
        for doc in Moleskine.app.documents:
            doc.reload_language()
        Moleskine.app.langs.save()
    
    def destroy(self, w, user_data=None):
        global langs_dialog
        langs_dialog = None
    
    def clicked(self, w, button, user_data=None):
        if button == 0:
            self.apply()
            self.close()
        elif button == 1:
            self.apply()
        else:
            self.close()

class FindDialog(GladeDialog):
    def __init__(self):
        GladeDialog.__init__(self, 'find_dialog')
        self.set_transient_for(Moleskine.app)
        self.editable_enters(self.find_combo.entry)
        self.connect('clicked', self.clicked)
        self.connect('destroy', self.destroy)
    
    def find(self, findfunc):
        findtools.flags = 0
        if self.case_check.get_active():
            findtools.flags = findtools.flags + GTKSCINTILLA.FIND_MATCH_CASE
        if self.words_check.get_active():
            findtools.flags = findtools.flags + GTKSCINTILLA.FIND_WHOLE_WORDS
        
        findtools.text = self.find_combo.entry.get_text()
        
        if not findfunc():
            from gnome.ui import GnomeMessageBox
            GnomeMessageBox('Cannot find the string `%s\'.' % findtools.text,
                    MESSAGE_BOX_INFO, STOCK_BUTTON_OK).run_and_close()
    
    def destroy(self, w, user_data=None):
        global find_dialog
        find_dialog = None
    
    def clicked(self, w, button, user_data=None):
        if button == 0:
            self.find(findtools.find_prev)
        elif button == 1:
            self.find(findtools.find_next)
        else:
            self.close()

class ReplaceDialog(GladeDialog):
    def __init__(self):
        GladeDialog.__init__(self, 'replace_dialog')
        self.set_transient_for(Moleskine.app)
        self.editable_enters(self.replace_combo.entry)
        self.connect('clicked', self.clicked)
        self.connect('destroy', self.destroy)
    
    def find(self, findfunc):
        findtools.flags = 0
        if self.case_check.get_active():
            findtools.flags = findtools.flags + GTKSCINTILLA.FIND_MATCH_CASE
        if self.words_check.get_active():
            findtools.flags = findtools.flags + GTKSCINTILLA.FIND_WHOLE_WORDS
        
        findtools.text = self.find_combo.entry.get_text()
        findtools.new_text = self.replace_combo.entry.get_text()
        
        if not findfunc():
            from gnome.ui import GnomeMessageBox
            GnomeMessageBox('Cannot find the string `%s\'.' % findtools.text,
                    MESSAGE_BOX_INFO, STOCK_BUTTON_OK).run_and_close()
    
    def replace(self):
        doc = Moleskine.app.get_current_document()
        findtools.new_text = self.replace_combo.entry.get_text()
        
        if self.all_check.get_active():
            ask = self.ask_check.get_active()
            doc.begin_undo_action()
            
            if doc.get_selection_start() == doc.get_selection_end():
                self.find(findtools.find_next)
            
            while doc.get_selection_start() != doc.get_selection_end():
                if ask:
                    from gnome.ui import GnomeMessageBox
                    
                    res = GnomeMessageBox(
                        'Do you want to replace this occurrence of the string?',
                        MESSAGE_BOX_QUESTION, STOCK_BUTTON_YES, STOCK_BUTTON_NO,
                        'Yes to all', STOCK_BUTTON_CANCEL).run_and_close()
                    if res == 0:
                        doc.replace_selection(findtools.new_text)
                    elif res == 1:
                        self.find(findtools.find_next)
                        continue
                    elif res == 2:
                        ask = FALSE
                    else:
                        break
                else:
                    doc.replace_selection(findtools.new_text)
                self.find(findtools.find_next)
            
            doc.end_undo_action()
        else:
            if doc.get_selection_start() != doc.get_selection_end():
                doc.replace_selection(findtools.new_text)
            self.find(findtools.find_next)
    
    def destroy(self, w, user_data=None):
        global replace_dialog
        replace_dialog = None
    
    def clicked(self, w, button, user_data=None):
        if button == 0:
            self.find(findtools.find_prev)
        elif button == 1:
            self.find(findtools.find_next)
        elif button == 2:
            self.replace()
        else:
            self.close()

def options():
    global options_dialog
    
    if options_dialog is None:
        options_dialog = OptionsDialog()
    else:
        options_dialog.get_window()._raise()

def languages():
    global langs_dialog
    
    if langs_dialog is None:
        langs_dialog = LanguagesDialog()
    else:
        langs_dialog.get_window()._raise()

def find():
    global find_dialog
    
    if find_dialog is None:
        find_dialog = FindDialog()
    else:
        find_dialog.get_window()._raise()

def replace():
    global replace_dialog
    
    if replace_dialog is None:
        replace_dialog = ReplaceDialog()
    else:
        replace_dialog.get_window()._raise()
