# Emacs.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2001   Michele Campeotto <micampe@f2s.com>
#
# Emacs keybindings for Moleskine by Matthew Pratt <mattpratt@yahoo.com>
# Please mail me if I haven't done your favourite emacs keystroke yet
#
# Copyright (c) 2001   Matthew Pratt <mattpratt@yahoo.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


# [Michele] Some thought about improving Emacs keybindings handling
#
# It would be good to have a structure like this:
#   emacs_keys = {
#       'C-x': {
#               'f': menu.open_document(),
#               's': menu.save_document(),
#               'c': menu.close_document(),
#               'C-c': menu.exit()
#       },
#       'C-_': menu.undo(),
#       'C-k': kill_line()
#   }
#
# and then when a key is pressed look into the structure to see how to
# proceed. If we find a dictionary, we grab the keyboard (don't ask me
# how) and wait for another keypress (or an ESC). If we find a function,
# we call it.
#
# This way we will have better Emacs emulation and easier customization
# of keybindings.
#
# Note: we also need to remove _all_ keybindings from Scintilla when
# Emacs emulation is activated.

import GDK

import Moleskine
import menu

last_keystroke = None

def emacs_keystroke(last, now, key1, key2):
    if last == None or now == None:
        return 0
    if not last.state & GDK.CONTROL_MASK \
            or not now.state & GDK.CONTROL_MASK:
        return 0
    if last.keyval == key1 and now.keyval == key2:
        return 1
    return 0

def key_press(w, event):
    global last_keystroke
    if event.state & GDK.CONTROL_MASK:
        if event.keyval == GDK.K or event.keyval == GDK.k:
            w.emit_stop_by_name('key_press_event')
            doc = Moleskine.app.get_current_document()
            pos = doc.get_current_pos()
            len = doc.line_length(doc.line_from_position(pos))
            doc.set_selection(pos, pos+len)
            doc.replace_selection('')
            return

        if event.keyval == GDK.underscore:
            menu.undo(None)
            last_keystroke = None
            return

        if emacs_keystroke(last_keystroke, event, GDK.x, GDK.s):
            menu.save_document(None);
            w.emit_stop_by_name('key_press_event')
            last_keystroke = None
            return

        if emacs_keystroke(last_keystroke, event, GDK.x, GDK.f):
            menu.open_document(None)
            last_keystroke = None
            return

        if emacs_keystroke(last_keystroke, event, GDK.x, GDK.c):
            menu.close_document(None)
            last_keystroke = None
            return

    last_keystroke = event
    return

##############################################################################
# We must redefine the menus, sice most of the default key bindings clash with
# emacs key strokes
#
# Idealy we would just change the bindings of the existing menu items but
# I dont know how to get to them
from gnome.ui import *
from gnome.uiconsts import *

file_menu = [
    UIINFO_ITEM_STOCK('_New', 'Create a new document',
                       menu.new_document,STOCK_MENU_NEW),
    UIINFO_ITEM_STOCK('_Open...', 'Load a file from disk',
                       menu.open_document, STOCK_MENU_OPEN),
    UIINFO_SUBTREE_STOCK('Open pre_vious', [], STOCK_MENU_BOOK_OPEN),
    UIINFO_SEPARATOR,
    UIINFO_MENU_SAVE_ITEM(menu.save_document),
    UIINFO_MENU_SAVE_AS_ITEM(menu.save_as_document),
    UIINFO_MENU_REVERT_ITEM(menu.revert_document),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK('Next', None, menu.file_next, STOCK_MENU_FORWARD),
    UIINFO_ITEM_STOCK('Previous', None, menu.file_previous, STOCK_MENU_BACK),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK('_Close', None, menu.close_document, STOCK_MENU_CLOSE),
    UIINFO_ITEM_STOCK('E_xit', None, menu.exit, STOCK_MENU_EXIT)
]

edit_menu = [
    (APP_UI_ITEM, '_Undo', 'Undo last action',
     menu.undo, None, APP_PIXMAP_STOCK, STOCK_MENU_UNDO,
     ord('_'), GDK.CONTROL_MASK),
    UIINFO_ITEM_STOCK('_Redo', 'Redo last action', menu.redo, STOCK_MENU_REDO),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, 'C_ut', 'Cut the current selection',
     menu.cut, None, APP_PIXMAP_STOCK, STOCK_MENU_CUT,
     ord('w'), GDK.CONTROL_MASK),
    UIINFO_ITEM_STOCK('_Copy', None, menu.copy, STOCK_MENU_COPY ),
    (APP_UI_ITEM, '_Paste', 'Paset from clipboard',
     menu.paste, None, APP_PIXMAP_STOCK, STOCK_MENU_PASTE,
     ord('y'), GDK.CONTROL_MASK),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, '_Find...', 'Find a string in current file',
     menu.find, None, APP_PIXMAP_STOCK, STOCK_MENU_SEARCH,
     ord('s'), GDK.CONTROL_MASK),
    (APP_UI_ITEM, 'Find _Next', 'Find next ocurrence of the string',
     menu.find_next, None, APP_PIXMAP_STOCK, STOCK_MENU_SEARCH,
     ord('g'), GDK.CONTROL_MASK),
    (APP_UI_ITEM, 'Find _Previous', 'Find previous ocurrence of the string',
     menu.find_previous, None, APP_PIXMAP_STOCK, STOCK_MENU_SEARCH,
     ord('g'), GDK.CONTROL_MASK + GDK.SHIFT_MASK),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, '_Replace...', 'Find and replace a string in current file',
     menu.replace, None, APP_PIXMAP_STOCK, STOCK_MENU_SRCHRPL,
     ord('r'), GDK.CONTROL_MASK),
    (APP_UI_ITEM, 'Replace Next',
     'Find and replace next string in current file',
     menu.replace_next, None, APP_PIXMAP_STOCK, STOCK_MENU_SRCHRPL,
     ord('e'), GDK.CONTROL_MASK),
]

settings_menu = [
    UIINFO_ITEM_STOCK('_Configuration', None, menu.options, STOCK_MENU_PROP),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK('Languages _settings...', None,
                      menu.lang_settings, STOCK_MENU_PREF),
    UIINFO_SUBTREE_STOCK('Change _language', [], STOCK_MENU_JUMP_TO)
]

tools_menu = [
    (APP_UI_ITEM, '_Next bookmark', None, menu.next_bookmark, None,
     APP_PIXMAP_STOCK, STOCK_MENU_FORWARD, 0, 0),
    (APP_UI_ITEM, '_Previous bookmark', None, menu.prev_bookmark, None,
     APP_PIXMAP_STOCK, STOCK_MENU_BACK, 0, 0),
    (APP_UI_ITEM, '_Toggle bookmark', None, menu.toggle_bookmark, None,
     APP_PIXMAP_STOCK, STOCK_MENU_BOOK_YELLOW, 0, 0),
    UIINFO_ITEM_STOCK('_Clear bookmarks', None,
                      menu.clear_bookmarks, STOCK_MENU_TRASH_FULL),
]

help_menu = [
    UIINFO_MENU_ABOUT_ITEM(menu.about)
]

main_menu = [
    UIINFO_SUBTREE('_File', file_menu),
    UIINFO_SUBTREE('_Edit', edit_menu),
    UIINFO_SUBTREE('_Settings', settings_menu),
    UIINFO_SUBTREE('_Tools', tools_menu),
    UIINFO_SUBTREE('_Help', help_menu)
]
