# Languages.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2001   Michele Campeotto <micampe@f2s.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from UserDict import *
import os.path
import string

from GTK import *
import gnome.config

import GTKSCINTILLA

_CONFIG_FILE = '/Moleskine/LanguageModes/'

Lexers = {
    'None'          : GTKSCINTILLA.LEXER_NULL,
    'Python'        : GTKSCINTILLA.LEXER_PYTHON,
    'C/C++'         : GTKSCINTILLA.LEXER_CPP,
    'HTML'          : GTKSCINTILLA.LEXER_HTML,
    'Perl'          : GTKSCINTILLA.LEXER_PERL,
    'SQL'           : GTKSCINTILLA.LEXER_SQL,
    'Visual Basic'  : GTKSCINTILLA.LEXER_VB,
    'LaTeX'         : GTKSCINTILLA.LEXER_LATEX,
    'LUA'           : GTKSCINTILLA.LEXER_LUA,
    'Pascal'        : GTKSCINTILLA.LEXER_PASCAL
}

Styles = {
    GTKSCINTILLA.LEXER_PYTHON : {
        'White space'               : GTKSCINTILLA.PYTHON_WHITE_SPACE,
        'Comment line'              : GTKSCINTILLA.PYTHON_COMMENT_LINE,
        'Number'                    : GTKSCINTILLA.PYTHON_NUMBER,
        'String double quote'       : GTKSCINTILLA.PYTHON_STRING_DOUBLE,
        'String single quote'       : GTKSCINTILLA.PYTHON_STRING_SINGLE,
        'Keyword'                   : GTKSCINTILLA.PYTHON_KEYWORD,
        'Long string double quote'  : GTKSCINTILLA.PYTHON_TRIPLE_DOUBLE,
        'Long string single quote'  : GTKSCINTILLA.PYTHON_TRIPLE_SINGLE,
        'Class name definition'     : GTKSCINTILLA.PYTHON_CLASS_STATEMENT,
        'Function name definition'  : GTKSCINTILLA.PYTHON_DEF_STATEMENT,
        'Operator'                  : GTKSCINTILLA.PYTHON_OPERATOR,
        'Identifier'                : GTKSCINTILLA.PYTHON_IDENTIFIER,
        'Comment block'             : GTKSCINTILLA.PYTHON_COMMENT_BLOCK,
        'String EOL'                : GTKSCINTILLA.PYTHON_STRING_EOL },
    GTKSCINTILLA.LEXER_CPP : {
        'White space'           : GTKSCINTILLA.CPP_WHITE_SPACE,
        'Comment block'         : GTKSCINTILLA.CPP_COMMENT_BLOCK,
        'Comment line'          : GTKSCINTILLA.CPP_COMMENT_LINE,
        'Comment doc'           : GTKSCINTILLA.CPP_COMMENT_DOC,
        'Number'                : GTKSCINTILLA.CPP_NUMBER,
        'Keyword'               : GTKSCINTILLA.CPP_KEYWORD,
        'String'                : GTKSCINTILLA.CPP_STRING,
        'Character'             : GTKSCINTILLA.CPP_CHARACTER,
        'UUID'                  : GTKSCINTILLA.CPP_UUID,
        'Preprocessor'          : GTKSCINTILLA.CPP_PREPROCESSOR,
        'Operator'              : GTKSCINTILLA.CPP_OPERATOR,
        'Identifier'            : GTKSCINTILLA.CPP_IDENTIFIER,
        'String not closed'     : GTKSCINTILLA.CPP_STRING_EOL,
        'Verbatim String (C#)'  : GTKSCINTILLA.CPP_STRING_VERBATIM },
    GTKSCINTILLA.LEXER_HTML : {
        'Text'                  : GTKSCINTILLA.HTML_TEXT,
        'Tag'                   : GTKSCINTILLA.HTML_TAG,
        'Unknown tag'           : GTKSCINTILLA.HTML_TAG_UNKNOWN,
        'Attribute'             : GTKSCINTILLA.HTML_ATTRIBUTE,
        'Unknown attribute'     : GTKSCINTILLA.HTML_ATTRIBUTE_UNKNOWN,
        'Number'                : GTKSCINTILLA.HTML_NUMBER,
        'String double quote'   : GTKSCINTILLA.HTML_DOUBLE_STRING,
        'String single quote'   : GTKSCINTILLA.HTML_SINGLE_STRING,
        'Other'                 : GTKSCINTILLA.HTML_OTHER,
        'Comment'               : GTKSCINTILLA.HTML_COMMENT,
        'Entity'                : GTKSCINTILLA.HTML_ENTITY,
        'XML tag ends (/>)'     : GTKSCINTILLA.HTML_XML_TAG_END,
        'Identifier start (<?)' : GTKSCINTILLA.HTML_XML_ID_START,
        'Identifier end (?>)'   : GTKSCINTILLA.HTML_XML_ID_END,
        'Script'                : GTKSCINTILLA.HTML_SCRIPT ,
        
        'JS Start'                  : GTKSCINTILLA.HTML_JS_START,
        'JS Default'                : GTKSCINTILLA.HTML_JS_DEFAULT,
        'JS Comment block'          : GTKSCINTILLA.HTML_JS_COMMENT_BLOCK,
        'JS Comment line'           : GTKSCINTILLA.HTML_JS_COMMENT_LINE,
        'JS Comment doc'            : GTKSCINTILLA.HTML_JS_COMMENT_DOC,
        'JS Number'                 : GTKSCINTILLA.HTML_JS_NUMBER,
        'JS Word'                   : GTKSCINTILLA.HTML_JS_WORD,
        'JS Keyword'                : GTKSCINTILLA.HTML_JS_KEYWORD,
        'JS String double quote'    : GTKSCINTILLA.HTML_JS_STRING_DOUBLE,
        'JS String single quote'    : GTKSCINTILLA.HTML_JS_STRING_SINGLE,
        'JS Symbol'                 : GTKSCINTILLA.HTML_JS_SYMBOL,
        'JS String EOL'             : GTKSCINTILLA.HTML_JS_EOL,
        
#        'PY Style 90'                   : GTKSCINTILLA.HTML_PY_90,
#        'PY Style 91'                   : GTKSCINTILLA.HTML_PY_91,
        'PY Comment line'               : GTKSCINTILLA.HTML_PY_COMMENT_LINE,
        'PY Number'                     : GTKSCINTILLA.HTML_PY_NUMBER,
        'PY String double quote'        : GTKSCINTILLA.HTML_PY_STRING_DOUBLE,
        'PY String single quote'        : GTKSCINTILLA.HTML_PY_STRING_SINGLE,
        'PY Keyword'                    : GTKSCINTILLA.HTML_PY_KEYWORD,
        'PY Long string double quote'   : GTKSCINTILLA.HTML_PY_TRIPLE_SINGLE,
        'PY Long string single quote'   : GTKSCINTILLA.HTML_PY_TRIPLE_DOUBLE,
        'PY Class name definition'      : GTKSCINTILLA.HTML_PY_CLASS_STATEMENT,
        'PY Function name definition'   : GTKSCINTILLA.HTML_PY_DEF_STATEMENT,
        'PY Operator'                   : GTKSCINTILLA.HTML_PY_OPERATOR,
        'PY Identifier'                 : GTKSCINTILLA.HTML_PY_IDENTIFIER,
        
        'PHP Default'               : GTKSCINTILLA.HTML_PHP_DEFAULT,
        'PHP String double quote'   : GTKSCINTILLA.HTML_PHP_STRING_DOUBLE,
        'PHP String single quote'   : GTKSCINTILLA.HTML_PHP_STRING_SINGLE,
        'PHP Keyword'               : GTKSCINTILLA.HTML_PHP_KEYWORD,
        'PHP Number'                : GTKSCINTILLA.HTML_PHP_NUMBER,
        'PHP Variable'              : GTKSCINTILLA.HTML_PHP_VARIABLE,
        'PHP Comment block'         : GTKSCINTILLA.HTML_PHP_COMMENT_BLOCK,
        'PHP Comment line'          : GTKSCINTILLA.HTML_PHP_COMMENT_LINE,
        'PHP String EOL'            : GTKSCINTILLA.HTML_PHP_EOL },
    GTKSCINTILLA.LEXER_PERL : {
        'White space'           : GTKSCINTILLA.PERL_WHITE_SPACE,
        'Error'                 : GTKSCINTILLA.PERL_ERROR,
        'Comment line'          : GTKSCINTILLA.PERL_COMMENT_LINE,
        'POD'                   : GTKSCINTILLA.PERL_POD,
        'Number'                : GTKSCINTILLA.PERL_NUMBER,
        'Keyword'               : GTKSCINTILLA.PERL_KEYWORD,
        'String'                : GTKSCINTILLA.PERL_STRING,
        'Character'             : GTKSCINTILLA.PERL_CHARACTER,
        'Punctuation'           : GTKSCINTILLA.PERL_PUNCTUATION,
        'Preprocessor'          : GTKSCINTILLA.PERL_PREPROCESSOR,
        'Operator'              : GTKSCINTILLA.PERL_OPERATOR,
        'Identifier'            : GTKSCINTILLA.PERL_IDENTIFIER,
        'Scalar'                : GTKSCINTILLA.PERL_SCALAR,
        'Array'                 : GTKSCINTILLA.PERL_ARRAY,
        'Hash'                  : GTKSCINTILLA.PERL_HASH,
        'Symbol table'          : GTKSCINTILLA.PERL_SYMBOL_TABLE,
        'Reference'             : GTKSCINTILLA.PERL_REF,
        'Regular expression'    : GTKSCINTILLA.PERL_REGEX,
        'Reg Subst'             : GTKSCINTILLA.PERL_REGSUBST,
        'Long quote'            : GTKSCINTILLA.PERL_LONG_QUOTE,
        'Backticks'             : GTKSCINTILLA.PERL_BACKTICKS,
        'Data section'          : GTKSCINTILLA.PERL_DATA_SECTION,
        'Here doc delimiter'    : GTKSCINTILLA.PERL_DATA_HERE_DELIM,
        'Here doc Q'            : GTKSCINTILLA.PERL_DATA_HERE_Q,
        'Here doc QQ'           : GTKSCINTILLA.PERL_DATA_HERE_QQ,
        'Here doc QX'           : GTKSCINTILLA.PERL_DATA_HERE_QX },
    GTKSCINTILLA.LEXER_SQL : {
        'White space'           : GTKSCINTILLA.SQL_WHITE_SPACE,
        'Comment block'         : GTKSCINTILLA.SQL_COMMENT_BLOCK,
        'Comment line'          : GTKSCINTILLA.SQL_COMMENT_LINE,
        'Comment doc'           : GTKSCINTILLA.SQL_COMMENT_DOC,
        'Number'                : GTKSCINTILLA.SQL_NUMBER,
        'Keyword'               : GTKSCINTILLA.SQL_KEYWORD,
        'String double quote'   : GTKSCINTILLA.SQL_DOUBLE_STRING,
        'String single quote'   : GTKSCINTILLA.SQL_SINGLE_STRING,
        'Symbols'               : GTKSCINTILLA.SQL_SYMBOLS,
        'Preprocessor'          : GTKSCINTILLA.SQL_PREPROCESSOR,
        'Operator'              : GTKSCINTILLA.SQL_OPERATOR,
        'Identifier'            : GTKSCINTILLA.SQL_IDENTIFIER,
        'String EOL'            : GTKSCINTILLA.SQL_STRING_EOL },
    GTKSCINTILLA.LEXER_VB : {
        'White space'       : GTKSCINTILLA.VB_WHITE_SPACE,
        'Comment block'     : GTKSCINTILLA.VB_COMMENT_BLOCK,
        'Comment line'      : GTKSCINTILLA.VB_COMMENT_LINE,
        'Comment doc'       : GTKSCINTILLA.VB_COMMENT_DOC,
        'Number'            : GTKSCINTILLA.VB_NUMBER,
        'Keyword'           : GTKSCINTILLA.VB_WORD,
        'String'            : GTKSCINTILLA.VB_STRING,
        'Character'         : GTKSCINTILLA.VB_CHARACTER,
        'UUID'              : GTKSCINTILLA.VB_UUID,
        'Preprocessor'      : GTKSCINTILLA.VB_PREPROCESSOR,
        'Operator'          : GTKSCINTILLA.VB_OPERATOR,
        'Identifier'        : GTKSCINTILLA.VB_IDENTIFIER,
        'String EOL'        : GTKSCINTILLA.VB_STRING_EOL },
    GTKSCINTILLA.LEXER_LATEX : {
        'White space'   : GTKSCINTILLA.LATEX_WHITE_SPACE,
        'Command'       : GTKSCINTILLA.LATEX_COMMAND,
        'Tag'           : GTKSCINTILLA.LATEX_TAG,
        'Math'          : GTKSCINTILLA.LATEX_MATH,
        'Comment'       : GTKSCINTILLA.LATEX_COMMENT },
    GTKSCINTILLA.LEXER_LUA : {
        'White space'       : GTKSCINTILLA.LUA_WHITE_SPACE,
        'Comment block'     : GTKSCINTILLA.LUA_COMMENT_BLOCK,
        'Comment line'      : GTKSCINTILLA.LUA_COMMENT_LINE,
        'Comment doc'       : GTKSCINTILLA.LUA_COMMENT_DOC,
        'Number'            : GTKSCINTILLA.LUA_NUMBER,
        'Keyword'           : GTKSCINTILLA.LUA_KEYWORD,
        'String'            : GTKSCINTILLA.LUA_STRING,
        'Character'         : GTKSCINTILLA.LUA_CHARACTER,
        'String literal'    : GTKSCINTILLA.LUA_LITERAL_STRING,
        'Preprocessor'      : GTKSCINTILLA.LUA_PREPROCESSOR,
        'Operator'          : GTKSCINTILLA.LUA_OPERATOR,
        'Identifier'        : GTKSCINTILLA.LUA_IDENTIFIER,
        'String EOL'        : GTKSCINTILLA.LUA_STRING_EOL },
    GTKSCINTILLA.LEXER_PASCAL : {
        'White space'           : GTKSCINTILLA.PASCAL_WHITE_SPACE,
        'Comment block'         : GTKSCINTILLA.PASCAL_COMMENT_BLOCK,
        'Comment line'          : GTKSCINTILLA.PASCAL_COMMENT_LINE,
        'Comment doc'           : GTKSCINTILLA.PASCAL_COMMENT_DOC,
        'Number'                : GTKSCINTILLA.PASCAL_NUMBER,
        'Keyword'               : GTKSCINTILLA.PASCAL_KEYWORD,
        'String double quote'   : GTKSCINTILLA.PASCAL_DOUBLE_STRING,
        'String single quote'   : GTKSCINTILLA.PASCAL_SINGLE_STRING,
        'Symbols'               : GTKSCINTILLA.PASCAL_SYMBOLS,
        'Preprocessor'          : GTKSCINTILLA.PASCAL_PREPROCESSOR }
}

class Language:
    def __init__(self, name, lexer):
        self.name = name
        if lexer is None:
            self.load()
        else:
            self.lexer = lexer
            self.init_styles()
    
    def init_styles(self):
        self.styles = {}
        if not Styles.has_key(self.lexer):
            self.styles[0] = {
                'font': None,
                'fore': None,
                'back': None }
            return
        for i in Styles[self.lexer].values():
            self.styles[i] = {
                'font': None,
                'fore': None,
                'back': None }
    
    def load(self):
        gnome.config.push_prefix(os.path.join(_CONFIG_FILE, self.name) + '/')
        
        self.lexer = gnome.config.get_int('lexer')
        self.keywords = [None, None, None, None, None]
        self.keywords[0] = gnome.config.get_string('keywords1')
        self.keywords[1] = gnome.config.get_string('keywords2')
        self.keywords[2] = gnome.config.get_string('keywords3')
        self.keywords[3] = gnome.config.get_string('keywords4')
        self.keywords[4] = gnome.config.get_string('keywords5')
        self.tab_size = gnome.config.get_int('tab_size')
        self.soft_tabs = gnome.config.get_bool('soft_tabs')
        self.indent_size = gnome.config.get_int('indent_size')
        self.show_indent = gnome.config.get_bool('show_indent')
        self.auto_indent = gnome.config.get_bool('auto_indent')
        self.edge_indicator = gnome.config.get_int('edge_indicator')
        self.edge_column = gnome.config.get_int('edge_column')
        self.edge_color = gnome.config.get_int('edge_color')
        
        self.load_styles()
        self.load_rules()
        
        gnome.config.pop_prefix()
    
    def load_rules(self):
        self.rules = []
        rules_count = gnome.config.get_int('rules_count')
        for i in range(rules_count):
            type = gnome.config.get_int('type_%i_type' % i)
            rule = gnome.config.get_string('type_%i_rule' % i)
            self.rules.append((type, rule))
    
    def load_styles(self):
        self.init_styles()
        
        if not Styles.has_key(self.lexer):
            return
        
        for style in Styles[self.lexer].values():
            s = self.styles[style]
            s['font'] = gnome.config.get_string('style_%i_font' % style)
            s['fore'] = gnome.config.get_string('style_%i_fore' % style)
            s['back'] = gnome.config.get_string('style_%i_back' % style)
    
    def save(self):
        gnome.config.push_prefix(os.path.join(_CONFIG_FILE, self.name) + '/')
        
        gnome.config.set_int('lexer', self.lexer)
        if self.keywords[0] is not None:
            gnome.config.set_string('keywords1', self.keywords[0])
        if self.keywords[1] is not None:
            gnome.config.set_string('keywords2', self.keywords[1])
        if self.keywords[2] is not None:
            gnome.config.set_string('keywords3', self.keywords[2])
        if self.keywords[3] is not None:
            gnome.config.set_string('keywords4', self.keywords[3])
        if self.keywords[4] is not None:
            gnome.config.set_string('keywords5', self.keywords[4])
        gnome.config.set_int('tab_size', self.tab_size)
        gnome.config.set_bool('soft_tabs', self.soft_tabs)
        gnome.config.set_int('indent_size', self.indent_size)
        gnome.config.set_bool('show_indent', self.show_indent)
        gnome.config.set_bool('auto_indent', self.auto_indent)
        gnome.config.set_int('edge_indicator', self.edge_indicator)
        gnome.config.set_int('edge_column', self.edge_column)
        gnome.config.set_int('edge_color', self.edge_color)
        
        self.save_styles()
        self.save_rules()
        
        gnome.config.pop_prefix()
    
    def save_rules(self):
        gnome.config.set_int('rules_count', len(self.rules))
        for i in range(len(self.rules)):
            gnome.config.set_int('type_%i_type' % i, self.rules[i][0])
            gnome.config.set_string('type_%i_rule' % i, self.rules[i][1])
    
    def save_styles(self):
        import Moleskine
        
        if not Styles.has_key(self.lexer):
            return
        
        for style in self.styles.keys():
            s = self.styles[style]
            if s['font'] is not None \
               and Moleskine.app.prefs['default style/font'] != s['font']:
                gnome.config.set_string('style_%i_font' % style, s['font'])
            if s['fore'] is not None \
               and Moleskine.app.prefs['default style/fore'] != s['fore']:
                gnome.config.set_string('style_%i_fore' % style, s['fore'])
            if s['back'] is not None \
               and Moleskine.app.prefs['default style/back'] != s['back']:
                gnome.config.set_string('style_%i_back' % style, s['back'])

class LanguagesList(UserDict):
    def new(self, name, lexer = None):
        self.data[name] = Language(name, lexer)
    
    def remove(self, name):
        del self.data[name]
    
    def load(self):
        gnome.config.push_prefix(_CONFIG_FILE)
        langs = gnome.config.get_string('Languages/languages')
        gnome.config.pop_prefix()
        
        if langs is not None:
            langs_list = string.split(langs, '\n')
        
        for lang_name in langs_list:
            self.new(lang_name)
        
        gnome.config.drop_all()
    
    def save(self):
        gnome.config.clean_file(_CONFIG_FILE)
        gnome.config.push_prefix(_CONFIG_FILE)
        langs = string.join(self.keys(), '\n')
        gnome.config.set_string('Languages/languages', langs)
        gnome.config.pop_prefix()
        
        for lang in self.values():
            lang.save()
        
        gnome.config.sync()
        gnome.config.drop_all()
