# menu.py
#
# Moleskine: a source code editor for the GNOME desktop
#
# Copyright (c) 2001   Michele Campeotto <micampe@f2s.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from gnome.uiconsts import *
import GDK

import Moleskine
import Dialogs
import findtools

def new_document(w, user_data=None):
    Moleskine.app.new_document()

def open_document(w, user_data=None):
    def ok_clicked(fs, user_data=None):
        Moleskine.app.open_document(fs.get_filename())
    Moleskine.app.ask_file(type = 'load', ok_cb = ok_clicked)

# FIXME: This function is *ugly*, but user_data is always None and
# I don't know how to get the label of a GtkRadioMenuItem :(
def open_recent(w, user_data=None):
    from gtk import GtkMenuItem, GtkMenu
    import copy
    
    item = GtkMenuItem(_obj = w)
    menu = item.get_ancestor(GtkMenu.get_type())
    app = Moleskine.app
    temp_list = copy.copy(app.files)
    if int(app.prefs['history/mru_sort']):
        temp_list.sort()
    menu_items = menu.children()
    if len(temp_list) < len(menu_items):
        menu_items = menu_items[1:]
    Moleskine.app.open_document(temp_list[menu_items.index(item)])
    
    del GtkMenuItem, GtkMenu

def save_document(w, user_data=None):
    app = Moleskine.app
    if app.get_current_document().filepath is None:
        save_as_document(w)
    else:
        app.save_document()

def save_as_document(w, user_data=None):
    def ok_clicked(fs, user_data=None):
        Moleskine.app.save_document(fs.get_filename())
    app = Moleskine.app
    app.ask_file(type = 'save', ok_cb = ok_clicked)

def revert_document(w, data=None):
    Moleskine.app.get_current_document().revert()

def file_next(w, user_data=None):
    app = Moleskine.app
    if app.notebook.get_current_page() == len(app.documents) - 1:
        app.notebook.set_page(0)
    else:
        app.notebook.next_page()

def file_previous(w, user_data=None):
    app = Moleskine.app
    if app.notebook.get_current_page() == 0:
        app.notebook.set_page(-1)
    else:
        app.notebook.prev_page()

def close_document(w, user_data=None):
    Moleskine.app.close_document()

def exit(w, user_data=None):
    from gtk import mainquit
    from gnome.ui import GnomeMessageBox
    
    for doc in Moleskine.app.documents:
        if not doc.can_close():
            res = GnomeMessageBox(
                    'There are unsaved files.\nDo you want to exit anyway?',
                    MESSAGE_BOX_QUESTION,
                    STOCK_BUTTON_YES, STOCK_BUTTON_NO).run_and_close()
            if res == 1 or res < 0:
                return 1
    Moleskine.app.save_prefs()
    Moleskine.app.save_mru()
    mainquit()

def undo(w, user_data=None):
    Moleskine.app.get_current_document().undo()

def redo(w, user_data=None):
    Moleskine.app.get_current_document().redo()

def cut(w, user_data=None):
    Moleskine.app.get_current_document().cut()

def copy(w, user_data=None):
    Moleskine.app.get_current_document().copy()

def paste(w, user_data=None):
    Moleskine.app.get_current_document().paste()

def find(w, user_data=None):
    Dialogs.find()

def find_next(w, user_data=None):
    findtools.find_next()

def find_previous(w, user_data=None):
    findtools.find_prev()

def replace(w, user_data=None):
    Dialogs.replace()

def replace_next(w, user_data=None):
    findtools.replace_next()

def options(w, user_data=None):
    Dialogs.options()

def lang_settings(w, user_data=None):
    Dialogs.languages()

# FIXME: This function is *ugly*, but user_data is always None and
# I don't know how to get the label of a GtkRadioMenuItem :(
def set_lang(w, user_data=None):
    from gtk import GtkRadioMenuItem, GtkMenu
    
    item = GtkRadioMenuItem(_obj = w)
    if not item.active:
        return
    menu = item.get_ancestor(GtkMenu.get_type())
    app = Moleskine.app
    doc = app.get_current_document()
    langs_names = app.langs.keys()
    langs_names.sort()
    menu_items = menu.children()
    if len(langs_names) < len(menu_items):
        menu_items = menu_items[1:]
    doc.set_language(app.langs[langs_names[menu_items.index(item)]])
    
    app.langs_menu = menu
    
    del GtkRadioMenuItem, GtkMenu

def next_bookmark(w, user_data=None):
    doc = Moleskine.app.get_current_document()
    line = doc.get_current_line()
    new_line = doc.marker_next(line + 1, 2)
    if new_line < 0:
        new_line = doc.marker_next(0, 2)
    if new_line >= 0:
        doc.go_to_line(new_line)

def prev_bookmark(w, user_data=None):
    doc = Moleskine.app.get_current_document()
    line = doc.get_current_line()
    new_line = doc.marker_previous(line - 1, 2)
    if new_line < 0:
        new_line = doc.marker_previous(doc.get_line_count(), 2)
    if new_line >= 0:
        doc.go_to_line(new_line)

def toggle_bookmark(w, user_data=None):
    doc = Moleskine.app.get_current_document()
    line = doc.get_current_line()
    if doc.marker_get(line) > 0:
        doc.marker_delete(line, 1)
    else:
        doc.marker_add(line, 1)

def clear_bookmarks(w, user_data=None):
    Moleskine.app.get_current_document().marker_delete_all(1)

def about(w, user_data=None):
    from about import show_infos
    show_infos()


file_menu = [
    (APP_UI_ITEM, '_New', 'Create a new document',
     new_document, None, APP_PIXMAP_STOCK, STOCK_MENU_NEW,
     GDK.n, GDK.CONTROL_MASK),
    (APP_UI_ITEM, '_Open...', 'Load a file from disk',
     open_document, None, APP_PIXMAP_STOCK, STOCK_MENU_OPEN,
     GDK.o, GDK.CONTROL_MASK),
    UIINFO_SUBTREE_STOCK('Open pre_vious', [], STOCK_MENU_BOOK_OPEN),
    UIINFO_SEPARATOR,
    UIINFO_MENU_SAVE_ITEM(save_document),
    (APP_UI_ITEM, 'Save _As...', 'Save the file to a different name',
     save_as_document, None, APP_PIXMAP_STOCK, STOCK_MENU_SAVE_AS,
     GDK.s, GDK.CONTROL_MASK + GDK.SHIFT_MASK),
    (APP_UI_ITEM, '_Revert', 'Reload the last saved version of the document',
     revert_document, None, APP_PIXMAP_STOCK, STOCK_MENU_REVERT,
     GDK.i, GDK.CONTROL_MASK),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, 'Next', None, file_next, None,
     APP_PIXMAP_STOCK, STOCK_MENU_FORWARD, GDK.period, GDK.CONTROL_MASK),
    (APP_UI_ITEM, 'Previous', None, file_previous, None,
     APP_PIXMAP_STOCK, STOCK_MENU_BACK, GDK.comma, GDK.CONTROL_MASK),
    UIINFO_SEPARATOR,
    UIINFO_MENU_CLOSE_ITEM(close_document),
    UIINFO_MENU_EXIT_ITEM(exit)
]

edit_menu = [
    UIINFO_MENU_UNDO_ITEM(undo),
    (APP_UI_ITEM, '_Redo', 'Redo last action',
     redo, None, APP_PIXMAP_STOCK, STOCK_MENU_REDO,
     GDK.z, GDK.CONTROL_MASK + GDK.SHIFT_MASK),
    UIINFO_SEPARATOR,
    UIINFO_MENU_CUT_ITEM(cut),
    UIINFO_MENU_COPY_ITEM(copy),
    UIINFO_MENU_PASTE_ITEM(paste),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, '_Find...', 'Find a string in current file',
     find, None, APP_PIXMAP_STOCK, STOCK_MENU_SEARCH,
     GDK.f, GDK.CONTROL_MASK),
    (APP_UI_ITEM, 'Find _Next', 'Find next ocurrence of the string',
     find_next, None, APP_PIXMAP_STOCK, STOCK_MENU_SEARCH,
     GDK.g, GDK.CONTROL_MASK),
    (APP_UI_ITEM, 'Find _Previous', 'Find previous ocurrence of the string',
     find_previous, None, APP_PIXMAP_STOCK, STOCK_MENU_SEARCH,
     GDK.g, GDK.CONTROL_MASK + GDK.SHIFT_MASK),
    UIINFO_SEPARATOR,
    (APP_UI_ITEM, '_Replace...', 'Find and replace a string in current file',
     replace, None, APP_PIXMAP_STOCK, STOCK_MENU_SRCHRPL,
     GDK.r, GDK.CONTROL_MASK),
    (APP_UI_ITEM, 'Replace Next',
     'Find and replace next string in current file',
     replace_next, None, APP_PIXMAP_STOCK, STOCK_MENU_SRCHRPL,
     GDK.e, GDK.CONTROL_MASK),
]

settings_menu = [
    UIINFO_ITEM_STOCK('_Configuration', None, options, STOCK_MENU_PROP),
    UIINFO_SEPARATOR,
    UIINFO_ITEM_STOCK('Languages _settings...', None,
                      lang_settings, STOCK_MENU_PREF),
    UIINFO_SUBTREE_STOCK('Change _language', [], STOCK_MENU_JUMP_TO)
]

tools_menu = [
    (APP_UI_ITEM, '_Next bookmark', None, next_bookmark, None,
     APP_PIXMAP_STOCK, STOCK_MENU_FORWARD, GDK.b, GDK.CONTROL_MASK),
    (APP_UI_ITEM, '_Previous bookmark', None, prev_bookmark, None,
     APP_PIXMAP_STOCK, STOCK_MENU_BACK,
     GDK.b, GDK.CONTROL_MASK + GDK.SHIFT_MASK),
    (APP_UI_ITEM, '_Toggle bookmark', None, toggle_bookmark, None,
     APP_PIXMAP_STOCK, STOCK_MENU_BOOK_YELLOW, GDK.b, GDK.MOD1_MASK),
    UIINFO_ITEM_STOCK('_Clear bookmarks', None,
                      clear_bookmarks, STOCK_MENU_TRASH_FULL),
]

help_menu = [
    UIINFO_MENU_ABOUT_ITEM(about)
]

main_menu = [
    UIINFO_SUBTREE('_File', file_menu),
    UIINFO_SUBTREE('_Edit', edit_menu),
    UIINFO_SUBTREE('_Settings', settings_menu),
    UIINFO_SUBTREE('_Tools', tools_menu),
    UIINFO_SUBTREE('_Help', help_menu)
]
